#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

from .trigger import *
from ..api.gitlab_api_queries import *
from ..arguments.arguments import *
from ..utils.option import *
from ..utils.file import *
from ..api.gitlab_api_job import *

labels_program = "trigger-squidient"
cobertura_job = "run:alya_staging"
cobertura_file = "cobertura.zip"

class TriggerSquidient(Trigger):

    def __init__(self, arguments):
        self._codecoverage = False
        self._cobertura = False
        super().__init__(arguments)
        for i in range(len(arguments)):
            if option(arguments[i], codecoverage):
                self._codecoverage = True
            elif option(arguments[i], cobertura):
                self._cobertura = True


    def process(self):
        r=self.trigger()
        if r != 0:
            exit(r)
        if self._wait :
            if self._codecoverage:
                self.coverage()
            if self._cobertura:
                self.cobertura()
            if self.is_success():
                print("Success!")
                exit(0)
            else:
                print("Failed!")
                exit(1)

    def coverage(self):
        try:
            print("Retrieving code coverage")
            cc = self._queries_wait.get(self._pipeline_object)["coverage"]
            write_text_file("cc.txt", cc)
            print("Code coverage has been found")
        except:
            print("Code coverage is unavailable")

    def cobertura(self):
        try:
            print("Retrieving cobertura files")
            jobs = self._queries_wait.get(self._pipeline_object, "jobs")
            for job in jobs:
                if job["name"] == cobertura_job:
                    job_id = job["id"]
            if job_id == - 1:
                raise Exception
            job = GitlLabAPIJob(job_id)
            self._queries_wait.download_job_artifacts(job, cobertura_file)
            print(command("unzip "+cobertura_file, output=True))
            print(command("ls -R *", output=True))
            print(command("tar -xf reports.tar.gz", output=True))
            print(command("ls -R reports/xml", output=True))

        except:
           print("Cobertura is unavailable")

    def is_success(self):
        return self._queries_wait.get(self._pipeline_object)["status"] == "success"

    def help(self, gazix="", program=""):
        print(gazix + " " + program + " [options]")
        print("\toptions:")
        print("\t-h/--help                Print the help")
        print("\t--gitlab [value]         Gitlab API url of the repository")
        print("\t--token_trigger [value]  Trigger token of the repository")
        print("\t--token_wait [value]     Token of the repository (required for wait, cc and cobertura)")
        print("\t--project [value]        ID of the repository")
        print("\t-v/--var [key]:[value]   Variable to transmit to the pipeline")
        print("\t-r/--ref [value]         Reference of the repository (main, master, etc.)")
        print("\t-cc                      Enable code coverage")
        print("\t--cobertura              Enable cobertura")
        print("\t-w/--wait                Wait for the end of the pipeline execution")
        print("\t-u/--unstable            Unstable mode")



#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

# -*- coding: utf-8 -*-

def read_text_file(file):
    f = open(file, 'r', errors="replace")
    data = f.read()
    f.close()
    return data


def write_text_file(file, data):
    f = open(file, 'w')
    f.write(data)
    f.close()




#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

from ..api.gitlab_api_access import *
from ..api.gitlab_api_queries import *
from ..arguments.arguments import *
from ..utils.option import *
from ..api.gitlab_api_MR import *

labels_program = "disable-automerge"


class DisableAutomerge:

    def __init__(self, arguments):
        self._help = False
        _gitlab_api_url = None
        _token = None
        _project = None
        self._MR = None
        for i in range(len(arguments)):
            if option(arguments[i], gitlab_api_url):
                _gitlab_api_url = arguments[i + 1]
                i += 1
            if option(arguments[i], token):
                _token = arguments[i + 1]
                i += 1
            elif option(arguments[i], project):
                _project = arguments[i + 1]
                i += 1
            elif option(arguments[i], MR):
                self._MR = arguments[i + 1]
                i += 1
            elif option(arguments[i], help_short, help_long):
                self._help = True
        self._access = GitLabAPIAccess(_gitlab_api_url, _token, _project)
        self._queries = GitLabAPIQueries(self._access)
        if self._help:
            self.help(gazix, labels_program)
            exit(0)
        if not self._access.is_valid():
            self.helpe(gazix, labels_program)
            exit(1)
        exit(self.disable())

    def disable(self):
        mr = GitlLabAPIMR(self._MR)
        print("Cancel the automatic merge of the Merge Request " + self._MR)
        self._queries.post(obj=mr, query="cancel_merge_when_pipeline_succeeds")
        return 0

    def help(self, gazix="", program=""):
        print(gazix + " " + program + " [options]")
        print("\toptions:")
        print("\t-h/--help                Print the help")
        print("\t--gitlab [value]         Gitlab API url of the repository")
        print("\t--token [value]          Token of the repository")
        print("\t--project [value]        ID of the repository")
        print("\t--MR [value]             MR IID of the repository")



#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

from .gitlab_api_object import *


class GitlLabAPILabel(GitLabAPIObject):

    def __init__(self, name, color, description=None, priority=None, id=-1):
        super().__init__(id)
        self._name = name
        self._color = color
        self._description = description
        self._priority = priority
        self.url()

    def get_type(self):
        return "labels"

    def url(self):
        self._url = "name=" + self._name + "&color=" + self._color
        if self._description is not None:
            self._url += "&description=" + self._description
        if self._priority is not None:
            self._url += "&priority=" + self._priority

    def get_url(self):
        return self._url

    def __str__(self):
        str = "label: " + self._name
        if id != -1:
            str += "\n\tid: " + self._id
        str += "\n\tcolor: " + self._color
        if self._description is not None:
            str += "\n\tdescription: " + self._description
        if self._priority is not None:
            str += "\n\tpriority: " + self._priority
        return str



#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################


from .gitlab_api_object import *

class GitlLabAPIJob(GitLabAPIObject):

    def __init__(self, id):
        super().__init__(id)

    def get_type(self):
        return "jobs"

    def get_url(self):
        return self._url

    def __str__(self):
        str = "job: " + str(self._id)
        return str


#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

# -*- coding: utf-8 -*-
import json
import os


def exist_json(name):
    """
       Test if Json file exist
    """
    return os.path.isfile(name)


def open_json(name):
    """
        Open a Json file.
    """
    with open(name) as json_data:
        d = json.load(json_data)
    return d


def read_json(string):
    return json.loads(string)


def save_json(d, name):
    with open(name, 'w') as fp:
        json.dump(d, fp, indent=4, sort_keys=True)



#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

import logging
import time
import os
import getpass

logging_context = "gazix"
#Initializing logger
#if not(os.path.isdir("gazix-log")):
#	os.system("mkdir gazix-log")
#os.system("rm -f gazix-log/*")

logger = logging.getLogger(logging_context)
date = time.strftime("%Y_%m_%d_%H")
user = getpass.getuser()
#fh = logging.FileHandler('gazix-log/'+date+'.log')

#Logger set debug level
logger.setLevel(logging.INFO)
#fh.setLevel(logging.DEBUG)

#Set logger format
formatter = logging.Formatter('[%(asctime)s] [%(funcName)s():%(lineno)d] %(levelname)s - %(message)s','%m-%d %H:%M:%S')
#fh.setFormatter(formatter)
#logger.addHandler(fh)
